unit janArrayButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,shellApi,
  ExtCtrls;


type
  TarrayButtonClicked= procedure (acol,arow:integer) of object;

  TjanArrayButton = class(TGraphicControl)
  private
    FptDown:TPoint;
    FPushDown: boolean;
    FColor: TColor;
    Frows: integer;
    Fcols: integer;
    FonArrayButtonClicked: TarrayButtonClicked;
    FCaptions: Tstringlist;
    FColors: TStringlist;
    FHints: TStringlist;
    procedure Setcols(const Value: integer);
    procedure Setrows(const Value: integer);
    procedure SetonArrayButtonClicked(const Value: TarrayButtonClicked);
    procedure SetCaptions(const Value: Tstringlist);
    procedure SetColors(const Value: TStringlist);
    procedure MouseToCell(const x, y: integer; var acol,arow: integer);
    function CellRect(acol,arow:integer): TRect;
    procedure SetHints(const Value: TStringlist);
    { Private declarations }
  protected
    { Protected declarations }
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    {Creates and initializes an instance of TjanArrayButton. }
    destructor Destroy; override;
    {Destroys an instance of TjanArrayButton.}
    procedure Paint; override;
    {Renders the image of the button.}
    procedure DoShowHint(var HintStr: string; var CanShow: Boolean;
      var HintInfo: THintInfo);
    {this procedure can be used in response to a application.onshowhint event
     button hints are stored in the hints property from arry top-left to array bottom right
     in your application create a seperate onshowhint event handler
     within that handler test hintinfo.hintcontrol is this object. If it is dispatch to this objects doShowHint.
     In the formcreate eventhandler include:
       application.OnShowHint:=drawhint;

     procedure TDrawF.DrawHint(var HintStr: string; var CanShow: Boolean;
     var HintInfo: THintInfo);
     begin
       if HintInfo.HintControl=janArrayButton1 then
          janArrayButton1.DoShowHint(Hintstr,canshow,hintinfo);
     end;

     I could have set the application.onshowhint handler directly in this component,
     but if you have more components that do this then only the last one would work
     }
  published
    { Published declarations }
    property Align;
    property rows:integer read Frows write Setrows;
    property cols:integer read Fcols write Setcols;
    property Font;
    property Captions:Tstringlist read FCaptions write SetCaptions;
    {A list of button captions from the top-left to the bottom-right button}
    property Hints:TStringlist read FHints write SetHints;
    {A list of button hints from the top-left to the bottom-right button}
    property Colors:TStringlist read FColors write SetColors;
    {A list of button colors from the top-left to the bottom-right button
     values must standard delphi color names like clred, clblue or hex color strings like $0000ff for red.
     please note the hex order in Delphi is BGR i.s.o. the RGB order you may know from HTML hex color triplets}
    property Hint;
    property ShowHint;
    property onArrayButtonClicked:TarrayButtonClicked read FonArrayButtonClicked write SetonArrayButtonClicked;
    {provides you with the column and row of the clicked button
    the topleft button has column=0 and row=0}
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('janBasic', [TjanArrayButton]);
end;

{ TjanArrayButton }



constructor TjanArrayButton.Create(AOwner: TComponent);
begin
  inherited;
  width := 35;
  height := 35;
  FColor := clSilver;
  FPushDown := false;
  FCols:=1;
  FRows:=1;
  showhint:=true;
  FCaptions:=TStringlist.create;
  FHints:=TStringlist.create;  
  FColors:=TStringlist.create;
end;

procedure TjanArrayButton.MouseToCell(const x,y:integer;var acol, arow:integer);
var
  dh,dw:integer;
begin
  dh:=(height-2) div FRows;
  dw:=(width-2) div Fcols;
  acol:=x div dw;
  arow:=y div dh;
end;

procedure TjanArrayButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var col,row:integer;
    dh,dw:integer;
begin
  if button=mbleft then
  begin
    FPushDown := true;
  {  dh:=(height-2) div FRows;
    dw:=(width-2) div Fcols;
    col:=x div dw;
    row:=y div dh;}
    MouseToCell(x,y,col,row);
    FptDown:=point(col,row);
    invalidate;
  end;
end;

procedure TjanArrayButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var od:TOpenDialog;
begin
  if button=mbleft then
  begin
    FPushDown := false;
    invalidate;
    if assigned(FonArraybuttonClicked) then
      onarraybuttonclicked(FptDown.x,FptDown.y);
  end
end;

procedure TjanArrayButton.Paint;
var R:TRect;
    col,row:integer;
    dh,dw:integer;
    x0,y0:integer;
    cap:string;
    backcolor:TColor;
    index:integer;

  procedure drawbackground(AColor: TColor);
  begin
      canvas.brush.color := AColor;
      canvas.FillRect(R);
  end;

  procedure drawup;
  begin
    drawbackground(Backcolor);
    Frame3D( Self.Canvas, R, clBtnHighlight, clBlack, 1 );
    if cap<>'' then
      drawtext(canvas.handle,pchar(cap),-1,R,DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;


  procedure drawdown;
  begin
    drawbackground(BackColor);
    Frame3D( Self.Canvas, R, clblack, clBtnHighlight, 1 );
    if cap<>'' then
      drawtext(canvas.handle,pchar(cap),-1,R,DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;



begin
  dh:=(height-2) div FRows;
  dw:=(width-2) div Fcols;
  for row:=0 to FRows-1 do
  begin
    y0:=1+row*dh;
    for col:=0 to FCols-1 do
    begin
      x0:=1+col*dw;
      R:=rect(x0,y0,x0+dw,y0+dh);
      index:=row*Fcols+col;
      if index<FCaptions.count then
        cap:=FCaptions[index]
      else
        cap:='';
      if index<FColors.count then
      try
       backcolor:=stringtocolor(FColors[index]);
       except
         backcolor:=clsilver;
       end
      else
       backcolor:=clsilver;
      if (csDesigning in ComponentState) then drawup
      else if (FptDown.x=col) and (FptDown.y=row) then
      begin
        if FPushDown then
          drawdown
        else
          drawup;
      end
      else drawup;
    end;
  end;
end;


destructor TjanArrayButton.Destroy;
begin
  FCaptions.free;
  FHints.Free;
  FColors.free;
  inherited;
end;


procedure TjanArrayButton.Setcols(const Value: integer);
begin
  if Fcols<>Value then
  begin
   if (value>=1) and (value<=10) then
   begin
     FCols:=value;
     invalidate;
   end
  end;
end;

procedure TjanArrayButton.Setrows(const Value: integer);
begin
  if FRows<>Value then
  begin
   if (value>=1) and (value<=10) then
   begin
     FRows:=value;
     invalidate;
   end
  end;
end;


procedure TjanArrayButton.SetonArrayButtonClicked(
  const Value: TarrayButtonClicked);
begin
  FonArrayButtonClicked := Value;
end;


procedure TjanArrayButton.SetCaptions(const Value: Tstringlist);
begin
  FCaptions.assign(Value);
  invalidate;
end;


procedure TjanArrayButton.CMFontChanged(var Message: TMessage);
begin
  canvas.font.Assign(font);
  invalidate;
end;

procedure TjanArrayButton.SetColors(const Value: TStringlist);
begin
  FColors.assign(Value);
  invalidate;
end;

function TjanArrayButton.CellRect(acol,arow:integer):TRect;
var
  dh,dw,x0,y0:integer;
  pt1,pt2:TPoint;
begin
  dh:=(height-2) div FRows;
  dw:=(width-2) div Fcols;
  y0:=1+arow*dh;
  x0:=1+acol*dw;
//  pt1:=clienttoscreen(point(x0,y0));
//  pt2:=clienttoscreen(point(x0+dw,y0+dh));
//  result:=rect(pt1.x,pt1.y,pt2.x,pt2.y);
  result:=rect(x0,y0,x0+dw,y0+dh);
end;

procedure TjanArrayButton.DoShowHint(var HintStr: string;
  var CanShow: Boolean; var HintInfo: THintInfo);
var acol,arow,x,y:integer;
    aname,s,ds:string;
    index:integer;
begin
  if HintInfo.HintControl = self then begin
    x:=HintInfo.CursorPos.x;
    y:=HintInfo.cursorPos.y;
    MouseToCell (x,y,acol,arow);
    if ((acol<0)or(arow<0)) then exit;
    index:=arow*Fcols+acol;
    if index<FHints.count then
      Hintstr:=FHints[index]
    else
      Hintstr:=Hint;
    Hintinfo.CursorRect :=CellRect (acol,arow);
    canshow:=true;
  end;
end;


procedure TjanArrayButton.SetHints(const Value: TStringlist);
begin
  FHints.assign(Value);
end;

end.

